
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/
//
// This file contains the JGOFS/DODS Surrogate library functions.
//

// $Log: Djdb.cc,v $
// Revision 1.20  1998/09/14 19:34:21  dan
// Modified get_values to pass correct temporary_string pointer declared in
// buf2val to the strcpy function.
//
// Removed get_initialRecord logic which was causing a segmentation fault
// after retrieving the first data record.
//
// Revision 1.19  1997/10/09 22:19:41  jimg
// Resolved conflicts in merge of 2.14c to trunk.
//
// Revision 1.18  1997/03/10 22:50:42  dan
// Modified request_das, request_data calls to explicitly set
// gui_p flag == true.
//
// Revision 1.17  1997/02/19 02:12:06  jimg
// Fixed calls to newest_dds().
//
// Revision 1.16  1997/01/31 18:30:57  dan
// Modified ParseConstraint to work correctly with Connect:request_das.
//
// Revision 1.14  1996/10/30 20:33:58  jimg
// JDBOpen modified to perform connect::request_das initially so that XDR* is not
// deleted improperly subsequent to connect::request_data call.
//
// Get_Record replaced so that deserialize is called once for each sequence instantiation
// rather than for each component of the sequence.
//
// Revision 1.10.2.2.2.1  1996/06/26 22:17:37  jimg
// Dans changes for 2.05.
//
// Revision 1.8  1995/09/18  22:49:53  jimg
// These files changed as a result of edit Dan made to the jg-dods code so that
// the servers use exiting jgofs methods (i.e., one set of jg-dods servers can
// be used to access N methods, without the linking hack previously used).
//
// Revision 1.7  1995/07/09  21:39:59  jimg
// Added copyright notice.
//
// Revision 1.6  1995/07/09  17:53:47  jimg
// Changed call to Connect::request_data so that asynchronous calls are used
// with jgofs.
//
// Revision 1.5  1995/03/14  02:12:11  jimg
// Fixed off-by-one error at end of Sequence.
// Removed code which was copied from original JGOFS source distribution.
// Much code placed in #ifdef NEVER ... #endif since it is never used, or
// was simplified.
//
// Revision 1.4  1995/02/16  18:52:22  dan
// Modified jdbopen to use DAS to populate the JGOFS data object's
// attribute data structure.
//
// Revision 1.3  1995/01/31  21:31:12  dan
// Removed files from repository which have been moved to ../netio
//
// Revision 1.1  1995/01/09  16:04:41  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.1  1994/10/25  9:00:00  dan
// First version of the jgofs surrogate library software.
//

#include "config_jg.h"

static char rcsid[]={"$Id: Djdb.cc,v 1.20 1998/09/14 19:34:21 dan Exp $"};

#include <assert.h>
#include <Pix.h>
#include <iostream.h>
#include <String.h>

#include "BaseType.h"
#include "JGSequence.h"
#include "JGConnect.h"
#include "Connections.h"

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif
#ifndef NULL
#define NULL 0
#endif


#define MAX_JG_OPEN 10
Connections<JGConnectPtr> conns(MAX_JG_OPEN);

#define HTSIZE 4096
#define MAX_BUF 2048
#define TOKEN 40
#define NUMNAMES 250
#define ATTRSIZE 1024
#define COMMENTSIZE 2048
#define MAX_UNITS 5

extern "C" {
  int jdbopen_(int*, char*, char*, int*, int*);
  int jdbreada_(int*, char*, int*);
  int jdbread_(int*, float*);
  int jdbcomments_(int*, char*);
  int jdblevel_(int*, int*);
  void jdbclose_(int*);
  int jdbattributes_(int*, int*, char*);
}

// I think that all the struct elements that are NUMNAMES sized arrays
// contain related information (i.e, vars[0] is the name of the zero-th
// variable, vLvls[0] is the level number of the zero-th variable, ...
//
// I think many of these fields are never used, and should be removed. In
// fact, this entire struct may be unecessary...
//
// 3/10/95 jhrg

struct OBJECT {
    int D_maxlevel,D_ntotal,initialRec;
    int D_lvlpntr[NUMNAMES];	// size was 11; changed to NUMNAMES. jhrg
    int D_rdpntr[NUMNAMES];
    char *vars[NUMNAMES];	// name of each variable in the sequence
    int vLvls[NUMNAMES];	// level of each variable inthe sequence
    char *vals[NUMNAMES];
    float fval[NUMNAMES];
    char D_attributes[NUMNAMES][ATTRSIZE], *attributeptr[NUMNAMES];
    char D_comments[COMMENTSIZE], *commentptr;
} *D_object[MAX_UNITS]={NULL,NULL,NULL,NULL,NULL};


// I think this returns the level of the variable VAR when FND is TRUE,
// otherwise it returns 0.
//
// This should be a mfunc of JGSequence (and possibly other types).
//
// 3/10/95 jhrg

int
max_level(JGSequence* seq, int currLvl, char *var, int *fnd)
{
    int lvl = 0;

    // Variable located in DDS
    for(Pix p = seq->first_var(); p; seq->next_var(p)) {
	if (seq->var(p)->name() == var) {
	    *fnd = TRUE; 
	    return lvl;
#ifdef NEVER
	    lvl = 0; 
	    break;                           // return currLvl.
#endif
	}							   
	else {
	    if (seq->var(p)->type_name() == "Sequence") {
		lvl = max_level((JGSequence *)seq->var(p), currLvl+1, var, 
				fnd);
		lvl++;
		if (*fnd == TRUE) 
		    return lvl;
#ifdef NEVER
		    break;
#endif
	    }
	}
    }

    return lvl;
}

// Given a JGSequence *SEQ, the number of variables N and the level of
// nesting CLVL, store the names and level of all variables in the current
// sequence *and* any varaibles in sub-sequences in VARNAMES and VARLVLS,
// resp. The initial values of N and CLVL are zero; for sebsequent recurrsive
// calls they are non-zero.
//
// Returns: the number of variables as well as their names and levels (via
// two value-result arrays).
//
// NB: This function should be re-written as a mfunc of the JGSequence
// class. That class should also contain extra information about each
// variable in a sequence that would replace the VARLVLS array used here. In
// addition, that class should have an interation operator that decends into
// sub-sequences automatically.
//
// jhrh 3/10/95

int
get_vars(JGSequence* seq, int n, int cLvl, char *varNames[], int varLvls[])
{
    int cN = n;

    for(Pix p = seq->first_var(); p; seq->next_var(p)) {
	if (seq->var(p)->type_name() == "Sequence") {
	    cN = get_vars((JGSequence *)seq->var(p), cN, cLvl+1, &varNames[0],
			 &varLvls[0]);
	}
	else {
	    strcpy(varNames[cN], seq->var(p)->name());
	    varLvls[cN] = cLvl;
	    cN++;
	}
    }
    return cN;
}

// I think this assumes that the Sequence contains only Strings (fine for
// JGOFS, but wrong in general).

int
get_values(JGSequence* seq, char *var, char *value)
{
    int status = FALSE;
    String tmp_str;
    String *tmp_str_ptr = &tmp_str; // We need a (void **) for read_val!

    for(Pix p = seq->first_var(); p; seq->next_var(p)) {
	if (seq->var(p)->type_name() == "Sequence") {
	  status = get_values((JGSequence *)seq->var(p), var, value);
	}
	else {
	    if (seq->var(p)->name() == var) {
	      status = seq->var(p)->buf2val((void **)&tmp_str_ptr);
	      strcpy(value, (const char *)(*tmp_str_ptr));
	      break;
	    }
	}
    }
    
    return status;
}

int
get_initialRecord(int *unit, char *values, int *valuesize)
{
    JGConnect *cnt = conns[*unit];
    struct OBJECT *obj = D_object[*unit];
    unsigned int found;

    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    // This code assumes that the data set contains a single variable at its
    // top level and that variable is a Sequence.
    for(int i = 0; i < obj->D_lvlpntr[obj->D_maxlevel]; i++) {
	found = FALSE;
	DDS *ndds = cnt->newest_dds();
	for(Pix p = ndds->first_var(); p; ndds->next_var(p))
	    switch(ndds->var(p)->type()) {
	      case dods_sequence_c:
		found = get_values((JGSequence *)ndds->var(p), 
				   obj->vars[i], values+(*valuesize)*i);
		break;
	    
	      default:
		cerr << "Error accessing variable: " << obj->vars[i] << endl;
	    }
    
	if (!found)
	    return -1;
    }

    return obj->D_maxlevel;
}

int
get_record(int *unit, char *values, int *valuesize)
{
    JGConnect *cnt = conns[*unit];
    struct OBJECT *obj = D_object[*unit];
    unsigned int found;

    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    XDR *where = cnt->source();

    // This code assumes that the data set contains a single variable at its
    // top level and that variable is a Sequence.

    DDS *ndds = cnt->newest_dds();
    for(Pix p = ndds->first_var(); p; ndds->next_var(p)) 
	found = ndds->var(p)->deserialize(where, ndds);

    if (!found) 
	return -1;

    for(int i = 0; i < obj->D_lvlpntr[obj->D_maxlevel]; i++) {
	found = FALSE;

	for(Pix p = ndds->first_var(); p; ndds->next_var(p))
	    switch (ndds->var(p)->type()) {
	      case dods_sequence_c:
		found = get_values((JGSequence *)ndds->var(p), 
				   obj->vars[i], values+(*valuesize)*i);
		break;
	      default:
		cerr << "Error accessing variable: " << obj->vars[i] << endl;
	    }
	
	if (!found)
	    return -1;
    }

    return obj->D_maxlevel;
}

void
ParseCStraint(String &name, String &cStraint)
{
    String projection = "", selection = "", tmpS = "";

    if (name.contains("(")) {
      cStraint = name.after("(");
      cStraint = cStraint.before(")");
      name = name.before("(");

      // ForEach clause in a multi-cStraint Jgofs parameter list
      // - extract the clause, and
      // - determine if its a projection or selection operation.
      //
      while ( cStraint.contains(",") > 0 )
	{
	  tmpS = cStraint.before(",");
	  cStraint = cStraint.after(",");

	  if (tmpS.contains("=") || tmpS.contains("<") || tmpS.contains(">")) 
	    if ( selection == "" ) selection = tmpS;
	    else selection = selection + "&" + tmpS;
	  else 
	    if ( projection == "" ) projection = tmpS;
	    else projection = projection + "," + tmpS;
	}
      // For a single-clause cStraint expression, or the last clause
      // of a multi-clause cStraint.
      //
      if (cStraint.contains("=") || cStraint.contains("<") || cStraint.contains(">")) {
	if ( selection == "" ) selection = cStraint;
        else selection = selection + "&" + cStraint;
      }
      else {
	if ( projection == "" ) projection = cStraint;
	else projection = projection + "," + cStraint;
      }
      
      // Construct the cStraint expression based on the parsed projection
      // and selection components.
      //
      if ( selection == "" ) 
	cStraint = projection;
      else {
	if ( projection == "" ) cStraint = selection;
	else cStraint = projection + "&" + selection;
      }
      name = name + "?" + cStraint;
      cStraint = "";
    }
}

int
jdbopen_(int* unit, char* objName, char* varNames, int* varSize, int* nVars)
{ 
    int maxLvls;
    String name = objName, cStraint = "";

    ParseCStraint(name,cStraint);

    JGConnect *cnt = new JGConnect(name, false);
    *unit = conns.add_connect(cnt);

#ifdef LOCAL
#error "LOCAL not handled"
#endif
    
    // If we are here, then the data set must be remote

    // find the first unused object
    int tUnit;
    for(tUnit = 0; tUnit < MAX_UNITS; tUnit++)
	if (!D_object[tUnit]) 
	    break;

    if ( tUnit >= MAX_UNITS ) {
	cerr << "Remote JDBOPEN_ Out of Units " << endl;
	return -999;
    }

    OBJECT *obj = new OBJECT;
    D_object[tUnit] = obj;
    *unit = tUnit;

    // Initialize the new object
#ifdef NEVER
    obj->htbufflen = 0;
    obj->htbuffptr = 0;
#endif
    *(obj->D_comments) = '\0';
    obj->commentptr = obj->D_comments;

    int i;
    for(i = 0; i < NUMNAMES; i++) {
	obj->D_rdpntr[i] = -1;
	obj->attributeptr[i] = obj->D_attributes[i];
    }
    obj->initialRec = 1;
    obj->D_ntotal = 0;
    obj->D_maxlevel = 0;

    // Retrieve the DataSet-Attribute-Structure from the Remote DODS Server.
    if ( !cnt->request_das( true ) ) {
      exit(1);
    }

    // Retrieve the DataSet-Descriptor-Structure from the Remote DODS Server
    // by initiating a data request. Data request is required in order to get
    // DDS associated with constrained data request. Otherwise, the Jgofs
    // open call would return the wrong variable information for subsequent
    // constrained data requests.

    // The first parameter is the cStraint string. The second parameter
    // (bool, true) causes the data request to be asynchronous.

    DDS *dds;
    if ( !(dds = cnt->request_data(cStraint, true)) ) {
      exit(1);
    }

    int Found, lvl, cLvl, totalV;
    int numD = *nVars;

    if (*nVars > 0) {
	for(i = 0; i < *nVars; ++i) {
	    obj->vars[i] = new char[TOKEN];
	    obj->vals[i] = new char[TOKEN];
	    obj->vLvls[i] = 0;
	}

	// Determine the level hierarchy
	// for variables supplied to the open call.
	for(i = 0; i < *nVars; ++i) {	    
	    lvl = 0;

	    for(Pix p = dds->first_var(); p; dds->next_var(p)) {
		Found = FALSE;
		BaseType *bPtr = dds->var(p);
		
		if (bPtr->name() == (varNames+(*varSize)*i)) {
		    maxLvls = 0; 
		    Found = TRUE;
		}
		else {
		    if (bPtr->type() == dods_sequence_c) {
			maxLvls = max_level((JGSequence *)bPtr, lvl+1,
					   (varNames+(*varSize)*i), &Found);
				//maxLvls++;
		    }
		}
		if (Found) {
		    obj->D_ntotal++;
		    obj->D_rdpntr[obj->D_ntotal-1] = i;
		    strcpy(obj->vars[i], (varNames+(*varSize)*i) );
		    obj->vLvls[i] = maxLvls;
		}
	    }
	}	
	obj->D_maxlevel = maxLvls;
    }
	
    //  No variables supplied to the open call.  Retrieve all variables
    //  from data-object, if number of variables exceeds absolute value
    //  of nVars, stored in 'numd', then return error;

    else { 
	*nVars = 0;
	for(i = 0; i < NUMNAMES; ++i) {
	    obj->vars[i] = new char[TOKEN];
	    obj->vals[i] = new char[TOKEN];
	    obj->vLvls[i] = 0;
	}

	// For each variable at each level in the data-object's hierarchy
	// increment the object's total variable count, 'obj->D_ntotal',
	// and store the variable identifier in the varNames string.

	lvl = -1;
	for(Pix p = dds->first_var(); p; dds->next_var(p)) {
	    BaseType *bPtr = dds->var(p);
		
	    if (bPtr->type() == dods_sequence_c) {
		*nVars = get_vars((JGSequence *)bPtr, *nVars, lvl+1, 
				  obj->vars, obj->vLvls);
	    }
	    else {
		strcpy(obj->vars[*nVars], bPtr->name());
		obj->vLvls[*nVars] = 0;
		(*nVars)++;
	    }
	} 

	totalV = *nVars; 
	*nVars = 0; 
	obj->D_ntotal = 0; 
	cLvl = 0;
	for (i = 0; i < totalV; ++i) {
	    if (totalV > -numD) {
		cout << "Too Many Variables." << endl;
	    }
	    else {
		(*nVars)++; obj->D_ntotal++;
		strcpy( (varNames+(*varSize)*((*nVars)-1)), obj->vars[i]);
		obj->D_rdpntr[obj->D_ntotal-1] = (*nVars)-1;
	    }
	    cLvl = ((obj->vLvls[i] > cLvl) ? obj->vLvls[i] : cLvl);
	}
	obj->D_maxlevel = cLvl;
    }

    // Successfully completed reading the variable names from the
    // Data-Descriptor-Structure.  Populate the Jgofs data-object's
    // Level-Pointer (D_lvlpntr) data structure.
    cLvl = obj->vLvls[0]; int tVar = 1;
    obj->D_lvlpntr[0] = 0; 

    for(i = 1; i < *nVars; ++i) {
	if (obj->vLvls[i] == cLvl) 
	    tVar++;
	else {
	    obj->D_lvlpntr[cLvl] = tVar;
	    cLvl = obj->vLvls[i]; tVar++;
	}
    }
    obj->D_lvlpntr[cLvl] = tVar;

    // For each variable at each level in the data-object's hierarchy
    // retrieve the variable's attribute information, and store
    // it in the JGOFS object's attribute data structure.
    //
    // NB: This code does not hanlde attribute vectors!
    // Switch from c-type strings to C++ String objects
    AttrTable *attr_table_ptr;

    for(i = 0; i < obj->D_ntotal; ++i) {
	attr_table_ptr = cnt->das().get_table(obj->vars[i]);
	if (attr_table_ptr) {
	    char *attr = obj->attributeptr[i]; // copy base pointer
	    *attr = '\0';
	    for(Pix p = attr_table_ptr->first_attr(); 
		p; attr_table_ptr->next_attr(p)) {
		strcat(attr, attr_table_ptr->get_name(p));
		strcat(attr, "\n");
		strcat(attr, attr_table_ptr->get_attr(p));
		strcat(attr, "\n");
	    }
	}
	else 
	    cerr << "JDBOPEN: Missing variable in DAS: " 
		 << obj->vars[i] << endl;
    }

    // Return the maximum depth needed to retrieve all of the variables
    // requested by the user's open call.

    return obj->D_maxlevel;
}

int 
jdbattributes_(int *unit, int *id, char *outcom)
{
    struct OBJECT *obj = D_object[*unit];

    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    char *sp = obj->attributeptr[*id];
    if (*sp) {
	int j = strcspn(sp, "\n");
	strncpy(outcom, sp, j);
	*(outcom + j) = '\0';
	obj->attributeptr[*id] += j + 1;
	return TRUE;
    } 
    else 
	return FALSE;
}

int 
jdblevel_(int *unit, int *varnum)
{
    struct OBJECT *obj = D_object[*unit];
  
    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    int j;
    for(j = 0; j < obj->D_ntotal; j++)
	if (*varnum == obj->D_rdpntr[j]) 
	    break;

    if(j >= obj->D_ntotal) 
	return -1;

    for(int i = 0; i <= obj->D_maxlevel; i++)
	if (j < obj->D_lvlpntr[i]) 
	    return i;

    return obj->D_maxlevel;
}

int 
jdbcomments_(int *unit, char *outcom)
{
    struct OBJECT *obj = D_object[*unit];
  
    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    if(*(obj->commentptr)) {
	int j = strcspn(obj->commentptr, "\n");
	strncpy(outcom, obj->commentptr, j);
	*(outcom+j)='\0';
	obj->commentptr += j+1;
	return 1;
    } 
    else 
	return 0;
}

void
jdbclose_(int *unit)
{
    struct OBJECT *obj = D_object[*unit];
  
    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return;
    }

    delete obj;
    D_object[*unit] = NULL;
}


int
jdbreada_(int *unit, char *values, int *valuesize)
{
    JGConnect *cnt = conns[*unit];
    struct OBJECT *obj = D_object[*unit];
    int status;

    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    // This implementation of JdbReadA attempts to mirror the behavior of the
    // JGOFS api behavior within the DODS design framework.
    //
    // Two new variables are added the JGOFS object data structure maintained
    // within the surrogate client library.  These variables maintain state
    // information concerning the current read level within a JGOFS sequence,
    // initialRecord and vals[obj->D_ntotal]. 
    // 
    // For the initial read, get the first record and return the value of
    // min-level. 

    if (obj->initialRec == TRUE) {
	//status = get_initialRecord(unit, values, valuesize);
	status = get_record(unit, values, valuesize);
	obj->initialRec = FALSE;
	if (status >= 0) {
	    return 0;
	}
	else 
	    return status;
    }
    else { 
	// For all subsequent reads, get the next record and iteratively test
	// for duplication at all but the deepest level. If the values at a
	// given level are identical to the previous read, continue to check
	// for duplicates on the next deeper level. When values for a given
	// level differ from the previous read, return that level value,
	// else, return the value of max-levels.
	//
	// To mimic the behavior of the original JGOFS read function, the
	// values from the previous read are temporarily stored for
	// subsequent testing for redundant level information.
	int i;
	char *cp;

	for(i = 0; i < obj->D_lvlpntr[obj->D_maxlevel-1]; i++ ) {
	    cp = values+(*valuesize)*i;      // remove leading spaces.
	    cp = cp + strspn(cp, " ");
	    strcpy(obj->vals[i], cp );
	}
	
	status = get_record(unit, values, valuesize);
	if ( status >= 0 ) {
	    for(i = 0; i < obj->D_lvlpntr[obj->D_maxlevel-1]; i++ ) {
		    cp = values+(*valuesize)*i;  // remove leading spaces.
		    cp = cp + strspn(cp, " ");
		    if ( strcmp(obj->vals[i],cp) != 0 ) 
			return obj->vLvls[i];
	    }
	    return obj->D_maxlevel;
	}
	else 
	    return status;
    }
}

int
jdbread_(int *unit, float values[])
{
    struct OBJECT *obj = D_object[*unit];

    if(!obj) {
	cerr << "Bad unit number: " << *unit << endl;
	return -1;
    }

    char vals[obj->D_ntotal * 80];
    int valSize = 80;
    int status = jdbreada_(unit, vals, &valSize);

    if ( status >= 0 ) {
	for(int i = 0; i < obj->D_ntotal; i++ ) {
	    char *cp = vals+(valSize*i);  // remove leading spaces.
	    cp = cp + strspn(cp, " ");
	    if( strspn(cp,"0123456789.+-") == 0 ) 
		values[i]= -9999.99;
	    else 
		values[i] = atof(cp);
	}
    }
    return status;
}
    





