
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/
// Implmentation for NCConnect: a subclass of Connect specialized for netcdf.
//
// jhrg 9/29/94

// $Log: JGConnect.cc,v $
// Revision 1.8  1997/03/10 22:52:21  dan
// Changed return type of JGConnect::request_data to pointer from
// reference.
//
// Revision 1.7  1997/02/19 02:13:16  jimg
// Fixed newest_dds() and request_dds() return type.
//
// Revision 1.6  1996/11/12 16:55:13  jimg
// Added config_jg.h include
//
// Revision 1.5  1996/09/19 20:55:24  jimg
// Simplified the implementation of JGConnect::request_data().
//
// Revision 1.4  1996/09/17 00:04:00  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.3.8.1.2.1  1996/06/26 22:17:44  jimg
// Dans changes for 2.05.
//
// Revision 1.3  1995/07/09  21:39:46  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  01:21:11  jimg
// Removed code copied over from the netcdf subclass of Connect.
//
// Revision 1.1  1995/01/09  16:03:32  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.7  1994/12/22  04:51:42  reza
// Updated to use DODS new named dimension capability.
//
// Revision 1.6  1994/11/23  20:59:52  reza
// Added heuristic dimension matching for Grid and Array classes.
// Added Dimension handling routines (ndims(), dim_size(), dim_name(),
// parse_dim(),and etc.)
//
// Revision 1.5  1994/11/18  21:33:33  reza
// Added _nvars and nvars() for number of variables in the file.
// Added _das_loc[] and parse_das_loc() to make a look-up table for variables
// in dds vs. das.
//
// Revision 1.4  1994/11/03  05:45:07  reza
// Moved error, request_das, and request _dds to the surrogate library for more
// flexibility and simpler error handling.
//
// Revision 1.3  1994/10/28  19:09:03  jimg
// Added _error, error() and fixed ncid() so that it retruns a reference.
//
// Revision 1.2  1994/10/05  20:23:30  jimg
// Fixed errors in *.h files comments - CVS bites again.
// Changed request_{das,dds} so that they use the field `_api_name'
// instead of requiring callers to pass the api name.
//
// Revision 1.1  1994/10/05  18:02:14  jimg
// First version of the connection management classes.
// This commit also includes early versions of the test code.
//

#include "config_jg.h"

static char rcsid[]={"$Id: JGConnect.cc,v 1.8 1997/03/10 22:52:21 dan Exp $"};

#ifdef __GNUG__
#pragma "implementation"
#endif

#include <assert.h>
#include "JGConnect.h"

JGConnect::JGConnect(const String &name, const bool verbose) : Connect(name, verbose)
{
    _jgid = -1;
}

JGConnect::~JGConnect()
{
}

int
JGConnect::jgid()
{
    return _jgid;
}

DDS *
JGConnect::newest_dds()
{
    return _newest_dds;
}

DDS *
JGConnect::request_data(const String expr, bool gui_p = true, 
			bool async = false, const String &ext = "dods")
{
    _newest_dds = Connect::request_data(expr, gui_p, async, ext);

    return _newest_dds;
}
