
// This is the main CGI program which extracts DAS class attributes from a
// netCDF data file. In addition, the attributes from an external file (if 
// exist, with the same name as the netCDF file and extension of .das at the 
// end) are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// ReZa 9/29/94
// DanH 10/26/94

// $Log: jg_das.cc,v $
// Revision 1.9  1998/03/20 00:29:59  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.8  1997/07/01 14:11:16  dan
// Modified handling of ancillary DAS files to make them work correctly.
//
// Added references to <strstream.h> to rework the way I handle String
// objects.
//
// Revision 1.7  1997/03/10 22:56:50  dan
// Added Error-handling.
//
// Revision 1.6  1996/11/12 16:59:15  jimg
// Added MIME error returns for some errors.
//
// Revision 1.5  1996/09/17 00:04:09  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:18:19  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:24  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:03  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:00  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:15:11  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.1  1995/01/09  16:04:46  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.2  1994/10/06  17:53:06  jimg
// Some reformatting of code for emacs, ...
// Fixed Makefile.in so that two targets are built: client and server.
// Fixed cast away of const in ncdas.cc.
//

static char rcsid[]={"$Id: jg_das.cc,v 1.9 1998/03/20 00:29:59 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <iostream.h>
#include <strstream.h>

char *getenv();
#include <unistd.h>
#include <pwd.h>

#include "DAS.h"
#include "cgi_util.h"

#include "jgofs.h"

extern bool create_das(DAS &das, char *filename, String *error);

int 
main(int argc, char * argv[])
{
    DAS das;

    if(argc < 1) {
	ErrMsgT("One parameter, filename, must be sent (JG_DAS)");
	set_mime_error(cout, HTERR_NO_CONTENT, 
		       "Internal error in DODS attribute server");
        exit(1);
    }

    // if external attribute file exist, read it ...
    // This file must be located in the same directory that 
    // the JGOFS data-object is located, and named the
    // JGOFS data-object filename with suffix '.dds'.

    String ext_file = argv[1];
    ext_file = ext_file + ".das";

    struct passwd *entry;
    entry = getpwnam("jgofs");	/* assume the name "jgofs" */

    String das_file =  entry->pw_dir;
    das_file = das_file + "/data/" + ext_file;

    FILE *in = fopen(das_file, "r");
    if (in) {
	int status = das.parse(in);

	if(!status) {

	  ostrstream oss;
	  oss << "jg_das: Parse error in external file: " << ext_file << ends;
	  String externalDasError = oss.str();
	  oss.freeze(0);

	  Error *ErrorObj = new Error(no_such_file, externalDasError);
	  set_mime_text(cout, dods_error);
	  ErrorObj->print(cout);
	
	  exit(1);
	}
	fclose(in);
    }

    else { // Open the JGOFS data-object and create a DataSet Attribute Structure

      String errMsg;
      
      if ( !create_das(das, argv[1], &errMsg) ) {      
	Error *ErrorObj = new Error(no_such_file, errMsg);
	
        set_mime_text(cout, dods_error);
	ErrorObj->print(cout);
	
	exit(1);
      }
    }

    // Forward the DAS class over the network to the client side

    set_mime_text(cout, dods_das);
    das.print(cout);

    return 0;                        
} 






