
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// This is the main CGI program which extracts DDS class attributes from a
// JGOFS data file. In addition, the attributes from an external file (if 
// exist, with the same name as the JGOFS data-object file with suffix '.dds')
// are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// DanH 10/21/94

// $Log: jg_dds.cc,v $
// Revision 1.10  1998/03/20 00:30:00  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.9  1997/06/06 03:54:13  jimg
// Changed call of create_dds() to the new version of that function.
//
// Revision 1.8  1997/03/27 18:29:30  jimg
// Update for version 2.13
//
// Revision 1.7  1997/03/10 23:01:37  dan
// Added DODS Error-handling.
//
// Revision 1.6  1996/11/12 17:01:15  jimg
// Added MIME error returns for certain errors.
//
// Revision 1.5  1996/09/17 00:04:10  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:18:20  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:26  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:04  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:02  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:15:13  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.1  1995/01/09  16:04:47  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.2  1994/10/06  17:53:06  jimg
// Some reformatting of code for emacs, ...
// Fixed Makefile.in so that two targets are built: client and server.
// Fixed cast away of const in ncdas.cc.

static char rcsid[]={"$Id: jg_dds.cc,v 1.10 1998/03/20 00:30:00 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <iostream.h>
#include <String.h>

#include "DDS.h"
#include "cgi_util.h"

extern bool create_dds(DDS &dds, String jg_name, String &error);

int 
main(int argc, char * argv[])
{
    DDS dds;

    if(argc < 2) {
	ErrMsgT("One parameter, filename, must be sent (jg_dds)");
	set_mime_error(cout, HTERR_NO_CONTENT, 
		       "Internal error in DODS attribute server");
        exit(1);
    }
    String dataset = argv[1];

    String errMsg;

    // Open the JGOFS data-object and create a DataSet Descriptor Structure

    if(!(create_dds(dds, argv[1], errMsg))){

	Error *ErrorObj = new Error(no_such_file, errMsg);

        set_mime_text(cout, dods_error);
	ErrorObj->print(cout);

	exit(1);
    }

    // if external DDS file exists, read it ...
    // This file must be in the same directory that 
    // the JGOFS data-object is located. Its name must be the
    // JGOFS data-object name with the '.dds' suffix.

    String ext_file = argv[1];
    ext_file = ext_file + ".dds";
    FILE *in = fopen(ext_file, "r");
 
    if (in) {
	int status; // = dds.parse(in);

	if(!status) {
	    String msg = "jg dds: Parse error in external file: " + ext_file;
	    ErrMsgT(msg);
	}
	fclose(in);
    }

    // Forward the DDS class over the network to the client side
    set_mime_text(cout, dods_dds);
    dds.print(cout);

    exit(0);                        
} 







