
// This is the main CGI program which serves JGOFS data using the DODS DAP. 
//
// DanH 12/20/94

// $Log: jg_dods.cc,v $
// Revision 1.14  1999/01/21 01:46:14  jimg
// Added call to set filename in the DDS. This is required for proper operation
// of the projection functions.
//
// Revision 1.13  1998/03/20 00:30:00  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.12  1997/08/05 22:45:01  jimg
// Changed type of option_char in `option_char = getopt(...)' from char to
// int. This fixes a bug that shows up on the SGI (where char is unsigned).
//
// Revision 1.11  1997/06/05 17:20:13  jimg
// Rewrote main() so that it uses functions added to cgi_util.cc for the 2.14
// release. These function help to unify the *_dods filters so that they all
// respond more uniformly.
//
// Revision 1.10  1997/03/13 06:12:26  jimg
// Fixed call to create_dds(). Now create_dds() returns an error message if an
// error was detected.
//
// Revision 1.9  1997/03/10 23:05:48  dan
// Modified code to reflect new sub-process changes used when
// compressing the data stream.
//
// Revision 1.7  1996/11/21 23:55:58  jimg
// Changed compression to new compressor function.
//
// Revision 1.6  1996/11/12 17:02:03  jimg
// Removed dead wood.
// Added MIME error returns for certain errors.
//
// Revision 1.5  1996/09/17 00:04:11  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.2.1.2.1  1996/06/26 22:18:21  jimg
// Dans changes for 2.05.
//
// Revision 1.2  1995/10/11  23:06:27  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.1  1995/09/29  23:31:56  jimg
// Added after much confusion and CVS screw-ups.
//
// Revision 1.1  1995/09/18  22:50:00  jimg
// These files changed as a result of edit Dan made to the jg-dods code so that
// the servers use exiting jgofs methods (i.e., one set of jg-dods servers can
// be used to access N methods, without the linking hack previously used).
//
// Revision 1.3  1995/03/14  02:15:14  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.2  1995/01/31  21:31:14  dan
// Removed files from repository which have been moved to ../netio
//

static char rcsid[]={"$Id: jg_dods.cc,v 1.14 1999/01/21 01:46:14 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <iostream.h>
#include <GetOpt.h>

#include "DDS.h"
#include "Sequence.h"
#include "Str.h"
#include "cgi_util.h"

#include "jgofs.h"
#include "JGSequence.h"
#include "JGStr.h"

extern char *name_path(const char *path);
extern bool create_dds(DDS &dds, String jg_name, String &error);

extern "C" {
    int ljdbreada_(int *, char (*)[VALSIZE], int *);
    int ljdbclose_(int *);
};

// This is called by do_version() (look in cgi_util.cc in the dap source
// directory). To enable the .ver extension to return information about the
// *dataset* version, flush out this function. NB: having the function return
// a null string causes do_version() to omit the information about the
// dataset version even if the users askes for it.

String
find_dataset_version(const String &dataset)
{
    return "";
}

int 
main(int argc, char * argv[])
{
    DDS dds;
    bool compression = false;
    bool version = false;
    int option_char;
    GetOpt getopt (argc, argv, "cv");

    while ((option_char = getopt()) != EOF)
	switch (option_char) {
	  case 'c': compression = true; break;
	  case 'v': version = true; break;
	  default:
	    usage(argv[0]); exit(1); break;
	}

    int next_arg = getopt.optind;

    if (version) {
	String script_ver = "";
	String dataset_ver = "";

	// Read the script version number if available.
	if(next_arg < argc) {
	    script_ver = argv[next_arg++];
	}

	// Read the dataset version number if the dataset name was given and a
	// version number is available.
	if(next_arg < argc) {
	    String dataset = argv[next_arg];
	    dataset_ver = find_dataset_version(dataset);
	}

	return do_version(script_ver, dataset_ver) ? 0: 1;
    }

    if(next_arg >= argc) {
	usage(argv[0]);
	exit(1);
    }
    
    String dataset = argv[next_arg++];

    dds.filename(dataset);

    // Open the JGOFS data-object and create a DataSet Descriptor Structure
    String errMsg;
    if (!create_dds(dds, dataset, errMsg)) {
	Error *ErrorObj = new Error(no_such_file, errMsg);

        set_mime_text(cout, dods_error);
	ErrorObj->print(cout);

	exit(1);
    }

    // Read ancillary DDS if it exists.
    if (!read_ancillary_dds(dds, dataset))
	return 1;

    // Read, serialize and send with CE
    if (do_data_transfer(compression, stdout, dds, dataset, ""))
	return 0;
    else
	return 1;
}
